package com.bwie.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Log;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LogRequest;
import com.bwie.common.result.PageResult;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.*;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author gxb
 * @description TODO
 * @date 2024-01-03 16:38
 */
@Service
public class EsServicelmpl implements EsService{
    @Autowired
    private RestHighLevelClient client;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private StringRedisTemplate redisTemplate;


    /**
     * 增量同步
     * @param data
     */
    @Override
    public void addBulk(List<Log> data) {
        BulkRequest bulkRequest = new BulkRequest();

        for (Log datum : data) {
            String s = JSON.toJSONString(datum);

            bulkRequest.add(new IndexRequest("log").id(datum.getLogId().toString())
                    .source(s, XContentType.JSON));
        }

        try {
            client.bulk(bulkRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 多条件列表
     * @param logRequest
     * @return
     */
    @Override
    public Result<List<Log>> loglist(LogRequest logRequest) {
        SearchRequest searchRequest = new SearchRequest("log");
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();


        //项目名称模糊查
        if (null!=logRequest.getItemName() && ""!=logRequest.getItemName()){
            MatchQueryBuilder name = QueryBuilders.matchQuery("itemName", logRequest.getItemName());
            boolQuery.must(name);
        }
        //项目客户方模糊查
        if (null!=logRequest.getItemCustomer() && ""!=logRequest.getItemCustomer()){
            MatchQueryBuilder Cusname = QueryBuilders.matchQuery("itemCustomer", logRequest.getItemCustomer());
            boolQuery.must(Cusname);
        }
        //区间
        RangeQueryBuilder rangeQuery = QueryBuilders.rangeQuery("itemApproval");

        if (null!=logRequest.getItemApproval()){
            RangeQueryBuilder gte = rangeQuery.gte(logRequest.getItemApproval().getTime());
            boolQuery.must(gte);
        }
         if (null!=logRequest.getItemApprovals()){
            RangeQueryBuilder lte = rangeQuery.lte(logRequest.getItemApprovals().getTime());
            boolQuery.must(lte);
        }
         //多角色登录
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        logRequest.setUserId(user.getUserId());
        //精确查询
        if (null!=logRequest.getUserId()){
            TermQueryBuilder id = QueryBuilders.termQuery("userId", logRequest.getUserId());
            boolQuery.must(id);
        }
        //设置高亮
        HighlightBuilder highlightBuilder = new HighlightBuilder();
        highlightBuilder.field("itemName");
        highlightBuilder.preTags("<span  style=\"color: red\">");
        highlightBuilder.postTags("</span>");

        sourceBuilder.highlighter(highlightBuilder);
        sourceBuilder.query(boolQuery);
        ArrayList<Log> logs = new ArrayList<>();
        searchRequest.source(sourceBuilder);
        try {
            SearchResponse search = client.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits hits = search.getHits();
            for (SearchHit hit : hits.getHits()) {
                String source = hit.getSourceAsString();
                Log log = JSON.parseObject(source, Log.class);

                //获取高亮
                Map<String, HighlightField> highlightFields = hit.getHighlightFields();
                HighlightField name = highlightFields.get("itemName");
                StringBuffer stringBuffer = new StringBuffer();
                if (null!=name){
                    for (Text fragment : name.getFragments()) {
                        stringBuffer.append(fragment);
                    }
                    log.setItemName(stringBuffer.toString());
                }
                logs.add(log);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return Result.success(logs,"Es列表");
    }
}
