package com.bwie.item.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.MqConstans;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.*;
import com.bwie.common.domain.request.Insertitem;
import com.bwie.common.domain.request.ItemRequest;
import com.bwie.common.domain.request.SendTel;
import com.bwie.common.domain.request.Update;
import com.bwie.common.exception.auth.AuthException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.item.mapper.ItemMapper;
import com.bwie.item.util.FastUtil;
import com.bwie.item.util.MsgUtil;
import org.springframework.amqp.rabbit.connection.CorrelationData;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.UUID;

/**
 * @author gxb
 * @description TODO
 * @date 2024-01-03 10:28
 */
@Service
public class ItemServicelmpl implements ItemService{
    @Autowired
    private ItemMapper itemMapper;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private FastUtil fastUtil;
    @Autowired
    private RabbitTemplate rabbitTemplate;


    @Override
    public Result ItemList(ItemRequest itemRequest) {
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        itemRequest.setRole(user.getRole());
        itemRequest.setUserId(user.getUserId());
        System.out.println("查询条件"+itemRequest);
        //查询列表
        List<Item> itemList = itemMapper.ItemList(itemRequest);
        return Result.success(itemList,"项目列表");
    }

    /**
     * 上传图片
     * @param myFile
     * @return
     */
    @Override
    public Result getImage(MultipartFile myFile) {
        String url = "";
        try {
            url = fastUtil.upload(myFile);
            if (url.equals("")){
                throw new AuthException("上传失败！！");
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return Result.success("http://43.143.161.183:8888/"+url,"上传成功~~");
    }

    /**
     * 添加
     * @param insertitem
     * @return
     */
    @Override
    public Result Insertitem(Insertitem insertitem) {
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        insertitem.setUserId(user.getUserId());

        //添加项目
        itemMapper.Insertitem(insertitem);
        //添加中间表

        //成本类型
        itemMapper.InsertType(insertitem);

        return Result.success("提交成功！！");
    }

    @Override
    public Result type() {
        List<Type> typeList = itemMapper.type();
        return Result.success(typeList);
    }

    /**
     * 获取当前登录人
     * @return
     */
    @Override
    public Result getInfo() {
        //获取当前登录人
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        return Result.success(user);
    }

    /**
     * 回显
     * @param itemId
     * @return
     */
    @Override
    public Result findBy(Integer itemId) {
        List<Middle> middleList = itemMapper.findBy(itemId);
        return Result.success(middleList);
    }

    /**
     * 修改
     * @param update
     * @return
     */
    @Override
    public Result upd(Update update) {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        //判断是否为审核员
        if (user.getRole()==2){
            SendTel sendTel = new SendTel();
            sendTel.setPhone(user.getTel());
            //修改前
            Middle middle = itemMapper.findMiddle(update.getMiddleId());
            itemMapper.upds(update);
            //修改后
            Middle middle02 = itemMapper.findMiddle(update.getMiddleId());
            //[项目名称]的估算类型[类型名称]金额未审核通过，金额：[估算金额]变更为[概算金额]
            String code = "项目编号：【"+middle02.getItemId()+"】，的估算类型【"+middle02.getTypeName()+"】" +
                    "金额未通过，金额：【"+middle.getFinallymoney()+"】变更为【"+update.getMiddleFinally()+"】";
            System.out.println("输出内容："+code);
            sendTel.setCode(code);
            rabbitTemplate.convertAndSend(MqConstans.SENDEXCHANGE,MqConstans.SENDTELKEY
            ,sendTel,message -> {
                message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
                return message;
                    },new CorrelationData(UUID.randomUUID().toString()));
        }
        itemMapper.upd(update);
        return Result.success("修改成功");
    }

    /**
     * 审核
     * @param middleId
     * @param state
     * @return
     */
    @Override
    public Result upState(Integer middleId, Integer state) {
       Integer i =  itemMapper.upState(middleId,state);
           //调用MQ添加日志
           String token = request.getHeader(TokenConstants.TOKEN);
           String userKey = JwtUtils.getUserKey(token);
           String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
           User user = JSON.parseObject(s, User.class);
           Middle middle = itemMapper.findMiddle(middleId);
           Log log = new Log();
           log.setItemId(middle.getItemId());
           log.setUserId(user.getUserId());

        /**
         * 审核日志
         */
        rabbitTemplate.convertAndSend(MqConstans.SENDCODEEXCHANGE,MqConstans.SENDCODEKEY,
                   log,message -> {
               message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
               return message;
                   },new CorrelationData(UUID.randomUUID().toString()));

        return Result.success("审核成功");
    }

    /**
     * 日志列表
     * @return
     */
    @Override
    public Result<List<Log>> LogList() {
        List<Log> logList = itemMapper.LogList();
        return Result.success(logList);
    }
}
