package com.com.bwie.mq.config;

import com.alibaba.nacos.shaded.org.checkerframework.checker.units.qual.C;
import com.bwie.common.constants.MqConstans;
import lombok.extern.log4j.Log4j;
import lombok.extern.log4j.Log4j2;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.connection.CorrelationData;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.web.bind.annotation.ExceptionHandler;

/**
 * @author gxb
 * @description TODO
 * @date 2023-12-22 17:19
 */
@Configuration
@Log4j2
public class MqConfig implements RabbitTemplate.ConfirmCallback,RabbitTemplate.ReturnsCallback {
    private RabbitTemplate rabbitTemplate;
    //消息转换
    @Bean
    public MessageConverter messageConverter(){
        return new Jackson2JsonMessageConverter();
    }
    //创建队列
    @Bean
    public Queue queue(){
        return new Queue(MqConstans.SENDCODEQUEUE,true);
    }
    //创建交换机
    @Bean
    public DirectExchange directExchange(){
        return new DirectExchange(MqConstans.SENDCODEEXCHANGE);
    }
    //绑定交换机和队列
    @Bean
    public Binding binding(){
        return BindingBuilder.bind(queue()).to(directExchange()).with(MqConstans.SENDCODEKEY);
    }

    @Primary
    @Bean
   public RabbitTemplate rabbitTemplate(ConnectionFactory connectionFactory){
        RabbitTemplate rabbitTemplate1 = new RabbitTemplate(connectionFactory);
        this.rabbitTemplate=rabbitTemplate1;
        rabbitTemplate.setMessageConverter(messageConverter());
        RabbitTemplate();
        return rabbitTemplate1;
    }
    //初始化
    public void RabbitTemplate(){
        rabbitTemplate.setConfirmCallback(this);
        rabbitTemplate.setReturnsCallback(this);
    }


    @Override
    public void confirm(CorrelationData correlationData, boolean b, String s) {
if (b){
   log.info("{}消息成功到达交换机",correlationData.getId());
}else {
    log.error("{}消息丢失",correlationData.getId());
}
    }

    @Override
    public void returnedMessage(ReturnedMessage returnedMessage) {
log.error("{}消息为进入队列",returnedMessage.getMessage().getMessageProperties().getMessageId());
    }
}
